import boto3
from infra import REGION, TABLE_NAME  # type: ignore[import-not-found]
from pydantic import BaseModel


class TableItem(BaseModel):
    user_id: str
    user_name: str
    email: str
    role: str
    full_name: str


def generate_table_items() -> list[TableItem]:
    items: list[TableItem] = []
    for i in range(100):
        if i % 2 == 0:
            role = "admin"
            full_name = "Bob Smith"
        else:
            role = "user"
            full_name = "Alice Doe"
        item = TableItem(
            user_id=f"user_{i}", user_name=f"user_{i}", email=f"user_{i}@gmail.com", role=role, full_name=full_name
        )
        items.append(item)
    return items


def populate_table() -> None:
    session = boto3.Session(region_name=REGION)
    table = session.resource("dynamodb").Table(TABLE_NAME)
    for item in generate_table_items():
        table.put_item(Item=item.model_dump())


if __name__ == "__main__":
    populate_table()



import os
from collections.abc import Generator
from pathlib import Path
from typing import Any, Literal

import boto3
import pytest
from faker import Faker
from moto import mock_aws
from mypy_boto3_dynamodb import DynamoDBClient
from mypy_boto3_dynamodb.type_defs import ExportTableToPointInTimeOutputTypeDef
from mypy_boto3_s3 import S3Client
from polyfactory import Use
from polyfactory.factories.pydantic_factory import ModelFactory
from polyfactory.pytest_plugin import register_fixture
from pydantic import BaseModel

from gurlon.dynamodb import DynamoTable

MOCK_EXPORT_ARN = "arn:aws:dynamodb:us-west-1:863881196012:table/gurlon-table/export/01732662110643-26e512e8"


class TableMetadata(BaseModel):
    table_name: str
    primary_key: str
    sort_key: str | None = None
    total_items: int


class TableItem(BaseModel):
    user_id: str
    user_name: str
    email: str
    role: str
    full_name: str


class TableMetadataFactory(ModelFactory[TableMetadata]):
    table_name = Use(ModelFactory.__faker__.name)


class TableItemFactory(ModelFactory[TableItem]):
    user_id = Use(ModelFactory.__faker__.uuid4)
    user_name = Use(ModelFactory.__faker__.user_name)
    email = Use(ModelFactory.__faker__.email)
    role = Use(ModelFactory.__faker__.random_element, elements=("admin", "user"))
    full_name = Use(ModelFactory.__faker__.name)


table_item_factory = register_fixture(TableItemFactory)
table_metadata_factory = register_fixture(TableMetadataFactory)


@pytest.fixture(autouse=True)
def mock_export_pitr(monkeypatch: pytest.MonkeyPatch, request: pytest.FixtureRequest) -> None:
    def mock_export_table_to_point_in_time(*args: Any, **kwargs: Any) -> str:
        resp: ExportTableToPointInTimeOutputTypeDef = {
            "ExportDescription": {
                "ExportArn": MOCK_EXPORT_ARN,
                "ExportStatus": "IN_PROGRESS",
            },
            "ResponseMetadata": {
                "RequestId": "12345678901234567890123456789012",
                "HTTPStatusCode": 200,
                "HTTPHeaders": {
                    "server": "Server",
                    "date": "Mon, 01 Nov 2021 00:00:00 GMT",
                    "content-type": "application/x-amz-json-1.0",
                    "content-length": "123",
                    "connection": "keep-alive",
                    "x-amzn-requestid": "12345678901234567890123456789012",
                },
                "RetryAttempts": 0,
            },
        }
        return resp["ExportDescription"]["ExportArn"]  # pyright: ignore[reportTypedDictNotRequiredAccess]

    if "no_mock_export_pitr" in request.keywords:
        return  # Don't monkeypatch if test has marker 'no_mock_export_pitr
    monkeypatch.setattr(DynamoTable, "export_to_s3", mock_export_table_to_point_in_time)


@pytest.fixture(autouse=True)
def faker_seed() -> Literal[1]:
    return 1


@pytest.fixture(autouse=True)
def seed_factories(faker: Faker, faker_seed: Literal[1]) -> None:
    ModelFactory.__faker__ = faker
    ModelFactory.__random__.seed(faker_seed)


@pytest.fixture
def table_items(table_item_factory: TableItemFactory) -> list[TableItem]:
    return table_item_factory.batch(10)


@pytest.fixture
def aws_creds() -> None:
    """Mocked AWS Credentials for moto."""
    os.environ["AWS_ACCESS_KEY_ID"] = "testing"
    os.environ["AWS_SECRET_ACCESS_KEY"] = "testing"  # noqa: S105
    os.environ["AWS_SECURITY_TOKEN"] = "testing"  # noqa: S105
    os.environ["AWS_SESSION_TOKEN"] = "testing"  # noqa: S105
    os.environ["AWS_DEFAULT_REGION"] = "us-east-1"


@pytest.fixture
def s3(aws_creds: None) -> Generator[S3Client, Any, None]:
    """
    Return a mocked S3 client
    """
    with mock_aws():
        yield boto3.client("s3", region_name="us-east-1")


@pytest.fixture
def s3_bucket(s3: S3Client) -> str:
    """
    Create a bucket for testing
    """
    bucket_name = "test-bucket"
    s3.create_bucket(Bucket=bucket_name)
    waiter = s3.get_waiter("bucket_exists")
    waiter.wait(Bucket=bucket_name)
    return bucket_name


@pytest.fixture
def dynamodb(aws_creds: None) -> Generator[DynamoDBClient, Any, None]:
    """
    Return a mocked DynamoDB client
    """
    with mock_aws():
        yield boto3.client("dynamodb", region_name="us-east-1")


@pytest.fixture
def dynamodb_table(dynamodb: DynamoDBClient) -> str:
    """
    Create a table for testing
    """
    table_name = "test-table"
    dynamodb.create_table(
        AttributeDefinitions=[
            {"AttributeName": "user_id", "AttributeType": "S"},
        ],
        TableName=table_name,
        KeySchema=[
            {"AttributeName": "user_id", "KeyType": "HASH"},
        ],
        BillingMode="PAY_PER_REQUEST",
        # Stream needs to be enabled in order to perform PITR export to S3
        StreamSpecification={
            "StreamEnabled": True,
            "StreamViewType": "NEW_AND_OLD_IMAGES",
        },
        TableClass="STANDARD",
        DeletionProtectionEnabled=False,
    )
    # Once table exists, enable PITR
    waiter = dynamodb.get_waiter("table_exists")
    waiter.wait(TableName=table_name)
    dynamodb.update_continuous_backups(
        TableName=table_name,
        PointInTimeRecoverySpecification={
            "PointInTimeRecoveryEnabled": True,
        },
    )
    return table_name


@pytest.fixture
def populated_table(dynamodb_table: str, table_items: list[TableItem]) -> str:
    table_name = dynamodb_table
    table = boto3.resource("dynamodb").Table(table_name)
    for item in table_items:
        table.put_item(Item=item.model_dump())
    assert table.item_count == len(table_items)
    return table_name


@pytest.fixture
def populated_bucket(s3_bucket: str) -> str:
    client = boto3.client("s3")
    prefix = "tests/data/mock_export_contents/"
    for root, _dirs, files in Path("tests/data/mock_export_contents").walk():
        for file in files:
            key = os.path.join(root, file).replace(prefix, "")
            client.upload_file(
                Filename=os.path.join(root, file),
                Bucket=s3_bucket,
                Key=key,
            )
    return s3_bucket



"""
This type stub file was generated by pyright.
"""

def json_serial(o):  # -> str | float | int | list[Any]:
    ...
def dumps(dct, as_dict=..., **kwargs):  # -> object | str:
    """Dump the dict to json in DynamoDB Format
    You can use any other simplejson or json options
    :param dct - the dict to dump
    :param as_dict - returns the result as python dict (useful for DynamoDB boto3 library) or as json sting
    :returns: DynamoDB json format.
    """
    ...

def object_hook(dct):  # -> datetime | str | list[Any] | float | int | set[Any] | None:
    """DynamoDB object hook to return python values"""
    ...

def loads(s: str | dict, as_dict: bool = False, *args, **kwargs):  # -> Any:
    """Loads dynamodb json format to a python dict.
    :param s - the json string or dict (with the as_dict variable set to True) to convert
    :returns python dict object
    """
    ...



# pyright: reportTypedDictNotRequiredAccess=false
# ruff: noqa: N815
from datetime import datetime
from pathlib import Path

import boto3
import structlog
from pydantic import BaseModel

log: structlog.stdlib.BoundLogger = structlog.get_logger()


class ManifestFile(BaseModel):
    itemCount: int
    md5Checksum: str
    etag: str
    dataFileS3Key: str


class ManifestSummary(BaseModel):
    version: str
    exportArn: str
    startTime: datetime
    endTime: datetime
    tableArn: str
    tableId: str
    exportTime: datetime
    s3Bucket: str
    s3Prefix: str
    s3SseAlgorithm: str
    s3SseKmsKeyId: str | None
    manifestFilesS3Key: str
    billedSizeBytes: int
    itemCount: int
    outputFormat: str


class DynamoExport(BaseModel):
    arn: str
    local_data_dir: Path
    local_data_files: list[Path]
    manifest_files: list[ManifestFile]
    manifest_summary: ManifestSummary


def _parse_manifest_summary(path: Path) -> ManifestSummary:
    with path.open("r") as f:
        manifest_summary = ManifestSummary.model_validate_json(f.read())
    return manifest_summary


def _parse_manifest_files(path: Path) -> list[ManifestFile]:
    manifest_files: list[ManifestFile] = []
    with path.open("r") as f:
        for line in f.readlines():
            manifest_files.append(ManifestFile.model_validate_json(line))  # noqa: PERF401
    return manifest_files


class S3Bucket:
    def __init__(self, bucket_name: str, aws_region: str = "us-east-1") -> None:
        self.bucket_name = bucket_name
        # NOTE: This requires AWS credentials to be present locally by user
        self.client = boto3.client("s3", region_name=aws_region)

    def download_export(self, download_dir: Path, table_export_arn: str, key_prefix: str) -> DynamoExport:
        # First, need to find the latest export in the bucket
        resp = self.client.list_objects_v2(Bucket=self.bucket_name, Prefix=f"{key_prefix}/AWSDynamoDB/")
        if "Contents" not in resp:
            raise ValueError("No exports found in bucket")
        manifest_key: str | None = None
        for obj in resp["Contents"]:
            if obj["Key"].endswith("manifest-summary.json"):
                manifest_key = obj["Key"]
                break
        if manifest_key is None:
            raise ValueError("No manifest-summary.json found in bucket")
        self.client.download_file(
            Bucket=self.bucket_name,
            Key=manifest_key,
            Filename=(download_dir / "manifest-summary.json").as_posix(),
        )
        log.info("Downloaded manifest-summary.json")
        # Download manifest-files.json
        self.client.download_file(
            Bucket=self.bucket_name,
            Key=manifest_key.replace("manifest-summary", "manifest-files", 1),
            Filename=(download_dir / "manifest-files.json").as_posix(),
        )
        log.info("Downloaded manifest-files.json")

        manifest_summary = _parse_manifest_summary(download_dir / "manifest-summary.json")
        manifest_files = _parse_manifest_files(download_dir / "manifest-files.json")
        local_data_files = []
        for file in manifest_files:
            filename = (download_dir / file.dataFileS3Key.split("/")[-1]).as_posix()
            log.debug("Downloading data file", key=file.dataFileS3Key, filename=filename)
            local_data_files.append(Path(filename))
            self.client.download_file(
                Bucket=self.bucket_name,
                Key=file.dataFileS3Key,
                Filename=filename,
            )
            log.info("Downloaded data file", key=file.dataFileS3Key, filename=filename)
        return DynamoExport(
            arn=table_export_arn,
            local_data_dir=download_dir,
            local_data_files=local_data_files,
            manifest_files=manifest_files,
            manifest_summary=manifest_summary,
        )



import gzip
from collections.abc import Generator
from pathlib import Path
from typing import Any

import duckdb
import orjson
import structlog
from dynamodb_json import json_util  # type: ignore
from pydantic import BaseModel
from sqlalchemy import Engine
from sqlmodel import Session, SQLModel, create_engine

from gurlon.dynamodb import DynamoTable
from gurlon.s3 import DynamoExport, S3Bucket

log: structlog.stdlib.BoundLogger = structlog.get_logger()


class DataExporter:
    def __init__(
        self,
        aws_region: str,
        table_name: str,
        bucket_name: str,
        table_export_arn: str | None = None,
        key_prefix: str = "gurlon",
    ) -> None:
        log.debug("Initializing DataExporter", aws_region=aws_region, table_name=table_name, bucket_name=bucket_name)
        self.aws_region = aws_region
        self.table: DynamoTable = DynamoTable(table_name, aws_region)
        self.bucket: S3Bucket = S3Bucket(bucket_name, aws_region)
        self.table_export_arn: str | None = table_export_arn
        self.key_prefix = key_prefix
        self.export_metadata: DynamoExport | None = None
        self.decompressed_files: list[Path] = []

    def export_data(self) -> str:
        log.debug("Exporting data to S3", table_name=self.table.table_name, bucket_name=self.bucket.bucket_name)
        # Export DynamoDB table data to S3
        self.table_export_arn = self.table.export_to_s3(self.bucket.bucket_name, self.key_prefix)
        return self.table_export_arn

    def download_data(self, download_dir: Path) -> Path:
        if not self.table_export_arn:
            raise ValueError("No export ARN found. Run export_data first")
        # Download data from S3
        download_dir.mkdir(exist_ok=True)
        log.debug(
            "Downloading data from S3",
            table_name=self.table.table_name,
            bucket_name=self.bucket.bucket_name,
            local_dir=download_dir,
        )
        self.export_metadata = self.bucket.download_export(download_dir, self.table_export_arn, self.key_prefix)
        # Uncompress the downloaded files
        self.decompress_data()
        # Combine the data into a single file
        combined_path = self.combine_data()
        # Optional: Validate the data
        # Save as CSV or other format
        log.info("Data downloaded, decompressed, and combined into a single file", combined_path=combined_path)
        return combined_path

    def decompress_data(self) -> None:
        if not self.export_metadata:
            raise ValueError("No export metadata found. Run download_data first")
        log.debug("Decompressing downloaded data", local_dir=self.export_metadata.local_data_dir)
        # Uncompress the downloaded files
        for data_file in self.export_metadata.local_data_files:
            log.debug("Decompressing file", file=data_file)
            with gzip.open(data_file.as_posix(), "rb") as f:
                content = f.read()
            decompressed_file = Path(data_file.as_posix().replace(".gz", ""))
            with decompressed_file.open("wb") as f:
                f.write(content)
            self.decompressed_files.append(decompressed_file)

    def _read_raw_data(self) -> Generator[str, Any, None]:
        for file in self.decompressed_files:
            log.debug("Reading raw data from file", file=file)
            with file.open("r") as f:
                lines = f.readlines()
            yield from lines

    def combine_data(self) -> Path:
        # Combine the data into a single file
        if self.decompressed_files == []:
            raise ValueError("No decompressed files found. Run decompress_data first")

        combined_data: list[dict[str, Any]] = []
        for row in self._read_raw_data():
            # Strip DynamoDB type markers from row
            item = json_util.loads(row)
            # Extract table data from the Item key
            combined_data.append(item["Item"])

        if self.export_metadata is None:
            raise ValueError("No local export metadata found")

        combined_data_path = self.export_metadata.local_data_dir / "combined_data.json"
        log.debug("Writing combined data to file", file=combined_data_path)

        with combined_data_path.open("wb") as f:
            f.write(orjson.dumps(combined_data, option=orjson.OPT_APPEND_NEWLINE))
        return combined_data_path


class DataTransformer:
    def __init__(self, combined_json_data: Path) -> None:
        self.combined_data = combined_json_data

    def to_parquet(self, output_path: Path | None = None) -> Path:
        if output_path:
            parquet_path = output_path
        else:
            parquet_path = self.combined_data.with_suffix(".parquet")
        rel = duckdb.read_json(self.combined_data.as_posix())
        rel.to_parquet(parquet_path.as_posix())
        return parquet_path

    def to_csv(self, output_path: Path | None = None) -> Path:
        if output_path:
            csv_path = output_path
        else:
            csv_path = self.combined_data.with_suffix(".csv")
        rel = duckdb.read_json(self.combined_data.as_posix())
        rel.to_csv(csv_path.as_posix())
        return csv_path

    def to_duckdb(self, output_path: Path | None = None, table_name: str = "data") -> Path:
        if output_path:
            duckdb_path = output_path
        else:
            duckdb_path = self.combined_data.with_suffix(".duckdb")
        con = duckdb.connect(duckdb_path.as_posix())
        con.execute(f"CREATE TABLE {table_name} AS SELECT * FROM read_json_auto('{self.combined_data.as_posix()}')")  # noqa: S608
        con.table(table_name).show()
        con.close()
        return duckdb_path

    def to_sqlite(self, model: BaseModel) -> None:
        pass

    def to_sqlmodel(self, model_cls: type[SQLModel], output_path: Path | None = None) -> Path:
        if output_path:
            sqlite_path = output_path
        else:
            sqlite_path = self.combined_data.with_suffix(".db")

        engine = self._create_sql_table(sqlite_path)
        with Session(engine) as session:
            self._populate_sql_table(model_cls, session)
        return sqlite_path

    def _create_sql_table(self, sqlite_path: Path) -> Engine:
        engine = create_engine(f"sqlite:///{sqlite_path.as_posix()}", echo=True)
        SQLModel.metadata.create_all(engine)
        return engine

    def _populate_sql_table(self, model: type[SQLModel], session: Session) -> None:
        # Read in the combined data using the path stored in self.combined_data
        with self.combined_data.open("rb") as f:
            table_items: list[dict[str, Any]] = orjson.loads(f.read())
        log.info("Read table items into memory", num_items=len(table_items))

        # Insert the data into the SQL table
        log.info("Iterating over table items and inserting into SQL table")
        for item in table_items:
            session.add(model(**item))

        session.commit()
        log.info("Data inserted into SQL table")



# pyright: reportTypedDictNotRequiredAccess=false
from datetime import datetime
from zoneinfo import ZoneInfo

import boto3
from botocore.exceptions import ClientError
from mypy_boto3_dynamodb.type_defs import DescribeTableOutputTypeDef
from pydantic import BaseModel


class TableMetadata(BaseModel):
    table_arn: str
    table_name: str
    table_status: str
    primary_key: str
    sort_key: str | None = None
    total_items: int
    total_size_bytes: int


def _build_table_metadata(data: DescribeTableOutputTypeDef) -> TableMetadata:
    table = data["Table"]
    try:
        return TableMetadata(
            table_arn=table["TableArn"],
            table_name=table["TableName"],
            table_status=table["TableStatus"],
            primary_key=table["KeySchema"][0]["AttributeName"],
            sort_key=table["KeySchema"][1]["AttributeName"] if len(table["KeySchema"]) > 1 else None,
            total_items=table["ItemCount"],
            total_size_bytes=table["TableSizeBytes"],
        )
    except (IndexError, KeyError) as e:
        raise ValueError("Invalid table metadata") from e


class DynamoTable:
    def __init__(self, table_name: str, region: str = "us-east-1") -> None:
        self.table_name = table_name
        # NOTE: This requires AWS credentials to be present locally by user
        self.client = boto3.client("dynamodb", region_name=region)
        self.metadata = self.get_metadata()

    def get_metadata(self) -> TableMetadata:
        response = self.client.describe_table(TableName=self.table_name)
        return _build_table_metadata(response)

    def export_to_s3(self, bucket: str, key_prefix: str) -> str:
        if len(key_prefix) < 3:
            raise ValueError("Key prefix must be at least 3 characters long")

        try:
            response = self.client.export_table_to_point_in_time(
                TableArn=self.metadata.table_arn,
                S3Bucket=bucket,
                ExportTime=datetime.now(ZoneInfo("UTC")),
                S3Prefix=key_prefix,
                ExportFormat="DYNAMODB_JSON",
                ExportType="FULL_EXPORT",
            )
        except ClientError as e:
            raise ValueError("Export failed") from e

        if response["ExportDescription"]["ExportStatus"] == "FAILED":
            raise ValueError("Export failed")

        return response["ExportDescription"]["ExportArn"]



import json
from pathlib import Path

from rich.console import Console
from rich.panel import Panel
from rich.syntax import Syntax
from sqlmodel import Field, SQLModel

from gurlon.processor import DataTransformer


class UserSqlModel(SQLModel, table=True):
    id: int | None = Field(default=None, primary_key=True)
    user_id: str
    user_name: str
    email: str
    role: str
    full_name: str


def log_output(*, parquet: Path, csv: Path, sql: Path, duckdb: Path) -> None:
    outputs = {
        "Parquet": parquet.as_posix(),
        "CSV": csv.as_posix(),
        "SQLite": sql.as_posix(),
        "DuckDB": duckdb.as_posix(),
    }
    syntax = Syntax(json.dumps(outputs, indent=2), "json", theme="lightbulb")
    console = Console()
    panel = Panel(syntax, title="[bold cyan]Data Transformation Output[/]", expand=False)
    console.print(panel)


def main() -> None:
    combined_data = Path.home() / "Downloads" / "dynamodb_exports" / "combined_data.json"
    if not combined_data.exists():
        raise FileNotFoundError(f"File not found: {combined_data}")
    transformer = DataTransformer(combined_data)
    parquet = transformer.to_parquet()
    csv = transformer.to_csv()
    sql = transformer.to_sqlmodel(UserSqlModel)
    duckdb = transformer.to_duckdb()
    log_output(parquet=parquet, csv=csv, sql=sql, duckdb=duckdb)


if __name__ == "__main__":
    main()



from gurlon.processor import DataExporter


def main() -> None:
    exporter = DataExporter("us-west-1", "gurlon-table", "gurlon-bucket")
    export_arn = exporter.export_data()
    print(f"Export ARN: {export_arn}")


if __name__ == "__main__":
    main()



import argparse
from typing import Literal

import boto3
from pydantic import BaseModel

REGION: Literal["us-west-1"] = "us-west-1"
TABLE_NAME = "gurlon-table"
BUCKET_NAME = "gurlon-bucket"


class ResourceInfo(BaseModel):
    name: str
    arn: str
    region: str


def create_dynamodb_table(session: boto3.Session) -> ResourceInfo:
    dynamodb = session.client("dynamodb", region_name=REGION)
    create_resp = dynamodb.create_table(
        AttributeDefinitions=[
            {"AttributeName": "user_id", "AttributeType": "S"},
        ],
        TableName=TABLE_NAME,
        KeySchema=[
            {"AttributeName": "user_id", "KeyType": "HASH"},
        ],
        BillingMode="PAY_PER_REQUEST",
        # Stream needs to be enabled in order to perform PITR export to S3
        StreamSpecification={
            "StreamEnabled": True,
            "StreamViewType": "NEW_AND_OLD_IMAGES",
        },
        TableClass="STANDARD",
        DeletionProtectionEnabled=False,
    )
    # Once table exists, enable PITR
    waiter = dynamodb.get_waiter("table_exists")
    waiter.wait(TableName=TABLE_NAME)
    dynamodb.update_continuous_backups(
        TableName=TABLE_NAME,
        PointInTimeRecoverySpecification={
            "PointInTimeRecoveryEnabled": True,
        },
    )
    # Validate we have the table ARN
    if "TableArn" not in create_resp["TableDescription"]:
        raise ValueError("Table ARN not found in response")
    return ResourceInfo(name=TABLE_NAME, arn=create_resp["TableDescription"]["TableArn"], region=REGION)


def create_s3_bucket(session: boto3.Session) -> ResourceInfo:
    s3 = session.client("s3", region_name=REGION)
    s3.create_bucket(Bucket=BUCKET_NAME, ACL="private", CreateBucketConfiguration={"LocationConstraint": REGION})
    waiter = s3.get_waiter("bucket_exists")
    waiter.wait(Bucket=BUCKET_NAME)
    return ResourceInfo(name=BUCKET_NAME, arn=f"arn:aws:s3:::{BUCKET_NAME}", region=REGION)


def log_resources(table: ResourceInfo, bucket: ResourceInfo) -> None:
    print(f"Created DynamoDB table: {table.name} ({table.arn}) in {table.region}")
    print(f"Created S3 bucket: {bucket.name} ({bucket.arn}) in {bucket.region}")


def create_infrastructure() -> None:
    session = boto3.Session(region_name=REGION)
    table = create_dynamodb_table(session)
    bucket = create_s3_bucket(session)
    log_resources(table, bucket)


def destroy_infrastructure() -> None:
    session = boto3.Session(region_name=REGION)
    table = session.client("dynamodb", region_name=REGION)
    table.delete_table(TableName=TABLE_NAME)
    bucket = session.client("s3", region_name=REGION)
    bucket.delete_bucket(Bucket=BUCKET_NAME)
    print(f"Deleted DynamoDB table: {TABLE_NAME}")
    print(f"Deleted S3 bucket: {BUCKET_NAME}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--operation", type=str, choices=["create", "destroy"], required=True)
    args = parser.parse_args()
    if args.operation == "create":
        create_infrastructure()
    elif args.operation == "destroy":
        destroy_infrastructure()
    else:
        raise ValueError("Invalid operation")



from pathlib import Path

import pytest
from sqlmodel import Field, SQLModel

from gurlon import processor
from tests.conftest import MOCK_EXPORT_ARN


def test_create_data_exporter_instance(populated_table: str, s3_bucket: str) -> None:
    exporter = processor.DataExporter(aws_region="us-east-1", table_name=populated_table, bucket_name=s3_bucket)
    assert exporter.aws_region == "us-east-1"
    assert exporter.table.table_name == populated_table
    assert exporter.bucket.bucket_name == s3_bucket


def test_export_to_s3(populated_table: str, s3_bucket: str) -> None:
    exporter = processor.DataExporter(aws_region="us-east-1", table_name=populated_table, bucket_name=s3_bucket)
    export_arn = exporter.export_data()
    assert export_arn is not None
    assert export_arn.startswith("arn:aws:dynamodb:")


def test_export_and_download(populated_table: str, populated_bucket: str, tmp_path: Path) -> None:
    exporter = processor.DataExporter(aws_region="us-east-1", table_name=populated_table, bucket_name=populated_bucket)
    export_arn = exporter.export_data()
    assert export_arn is not None
    assert export_arn == MOCK_EXPORT_ARN
    download_dir = tmp_path / "dynamodb_exports"
    download_dir.mkdir()
    combined_path = exporter.download_data(download_dir)
    assert combined_path.exists() is True
    assert combined_path.is_file()
    assert combined_path.suffix == ".json"
    assert combined_path.stat().st_size > 0
    assert combined_path.parent.name == "dynamodb_exports"
    assert combined_path.name == "combined_data.json"


@pytest.fixture
def combined_data() -> bytes:
    with Path("tests/data/expected/combined_data.json").open("rb") as input:
        return input.read()


@pytest.fixture
def combined_data_path(tmp_path: Path, combined_data: bytes) -> Path:
    data_path = tmp_path / "combined_data.json"
    with data_path.open("wb") as output:
        output.write(combined_data)
    return data_path


def test_transform_data_csv(combined_data_path: Path, tmp_path: Path) -> None:
    transformer = processor.DataTransformer(combined_data_path)
    output_path = tmp_path / "transformed_data.csv"
    csv_path = transformer.to_csv(output_path)
    assert csv_path.exists() is True
    assert csv_path.is_file()
    assert csv_path.suffix == ".csv"
    assert csv_path.stat().st_size > 0
    assert output_path == csv_path


def test_transform_data_parquet(combined_data_path: Path, tmp_path: Path) -> None:
    transformer = processor.DataTransformer(combined_data_path)
    output_path = tmp_path / "transformed_data.parquet"
    parquet_path = transformer.to_csv(output_path)
    assert parquet_path.exists() is True
    assert parquet_path.is_file()
    assert parquet_path.suffix == ".parquet"
    assert parquet_path.stat().st_size > 0
    assert output_path == parquet_path


def test_transform_data_sqlmodel(combined_data_path: Path, tmp_path: Path) -> None:
    class UserSqlModel(SQLModel, table=True):
        id: int | None = Field(default=None, primary_key=True)
        user_id: str
        user_name: str
        email: str
        role: str
        full_name: str

    transformer = processor.DataTransformer(combined_data_path)
    output_path = tmp_path / "gurlon-sqlite.db"
    sqlite_path = transformer.to_sqlmodel(UserSqlModel, output_path)
    assert sqlite_path.exists() is True
    assert sqlite_path.is_file()
    assert sqlite_path.suffix == ".db"
    assert sqlite_path.stat().st_size > 0
    assert output_path == sqlite_path


def test_transform_data_duckdb(combined_data_path: Path, tmp_path: Path) -> None:
    transformer = processor.DataTransformer(combined_data_path)
    output_path = tmp_path / "transformed_data.duckdb"
    duckdb_path = transformer.to_csv(output_path)
    assert duckdb_path.exists() is True
    assert duckdb_path.is_file()
    assert duckdb_path.suffix == ".duckdb"
    assert duckdb_path.stat().st_size > 0
    assert output_path == duckdb_path



from pathlib import Path

from gurlon.processor import DataExporter


def main() -> None:
    exporter = DataExporter(
        "us-west-1",
        "gurlon-table",
        "gurlon-bucket",
        "arn:aws:dynamodb:us-west-1:863881196012:table/gurlon-table/export/01732662110643-26e512e8",
    )
    download_dir = Path.home() / "Downloads" / "dynamodb_exports"
    exporter.download_data(download_dir=download_dir)


if __name__ == "__main__":
    main()



import pytest

from gurlon import dynamodb


def test_create_dynamodb_table_instance(populated_table: str) -> None:
    table = dynamodb.DynamoTable(table_name=populated_table)
    assert table.table_name == populated_table
    assert table.metadata is not None
    assert table.metadata.primary_key == "user_id"
    assert table.metadata.total_items > 0


def test_export_to_s3(populated_table: str, s3_bucket: str) -> None:
    table = dynamodb.DynamoTable(table_name=populated_table)
    export_arn = table.export_to_s3(bucket=s3_bucket, key_prefix="gurlon")
    assert export_arn is not None
    assert export_arn.startswith("arn:aws:dynamodb:")


@pytest.mark.parametrize("key_prefix", ["", "a", "24"])
@pytest.mark.no_mock_export_pitr
def test_export_to_s3_invalid_key_prefix(populated_table: str, s3_bucket: str, key_prefix: str) -> None:
    table = dynamodb.DynamoTable(table_name=populated_table)
    with pytest.raises(ValueError, match="Key prefix must be at least 3 characters long"):
        table.export_to_s3(bucket=s3_bucket, key_prefix=key_prefix)
